//
// G B A . H
//
// Include header file for GCC-GBA programming
//
// Apr. 30, 2003 by Wataru Nishida (http://www.skyfree.org)
//

#ifndef _GBA_H_
#define _GBA_H_

typedef unsigned short hword;		// Half word (2 bytes)
typedef unsigned char  byte;		// Byte

#define BG_PALETTE	0x05000000	// Background palette RAM
#define OBJ_PALETTE	0x05000200	// Object palette RAM
#define VRAM		0x06000000	// Video RAM frame buffer
#define VRAM_CHAR(n)	(VRAM + n * 0x4000)	// Character data base address
#define VRAM_SCRN(n)	(VRAM + n * 0x0800)	// Screen data base address

#define LCD_WIDTH	240		// 240 dots / horizontal line
#define LCD_HEIGHT	160		// 160 dots / vertical line
#define LCD_RWIDTH	30		// Real width in tile
#define LCD_RHEIGHT	20		// Real height in tile
#define LCD_VWIDTH	32		// Virtual width in tile
#define LCD_VHEIGHT	32		// Virtual height in tile
#define LCD_TILE_BYTE	(8 * 8)		// Byte size of single tile
#define LCD_TILE_HWORD	(8 * 4)		// Half word size of single tile

#define LCD_PAL_COLORS	256		// Total color number per palette
#define BGR(r, g, b)	(((b) << 10) + ((g) << 5) + (r))

#define register(p)	*((hword*) p)	// I/O register handling macro
#define IOBASE		0x04000000	// I/O register base address
#define LCD_CTRL	(IOBASE + 0)	// LCD control
#define   LCD_BG0EN	0x0100		//   Enable BG0
#define   LCD_BG1EN	0x0200		//   Enable BG1
#define   LCD_BG2EN	0x0400		//   Enable BG2
#define   LCD_BG3EN	0x0800		//   Enable BG3
#define   LCD_OBJEN	0x1000		//   Enable OBJ
#define   LCD_OBJMAP2D	0		//   OBJ 2D mapping
#define   LCD_OBJMAP1D	0x0040		//   OBJ 1D mapping
#define   LCD_MODE0	0		//   Video mode 0
#define   LCD_MODE1	1		//   Video mode 1
#define   LCD_MODE2	2		//   Video mode 2
#define   LCD_MODE3	3		//   Video mode 3
#define   LCD_MODE4	4		//   Video mode 4
#define   LCD_MODE5	5		//   Video mode 5
#define LCD_BG0		(IOBASE + 8)	// LCD BG0 control
#define LCD_BG1		(IOBASE + 10)	// LCD BG1 control
#define LCD_BG2		(IOBASE + 12)	// LCD BG2 control
#define LCD_BG3		(IOBASE + 14)	// LCD BG3 control
#define   LCD_SIZE3232	0x0000		//   BG size 32 x 32 tiles
#define   LCD_SIZE6432	0x4000		//   BG size 64 x 32 tiles
#define   LCD_SIZE3264	0x8000		//   BG size 32 x 64 tiles
#define   LCD_SIZE6464	0xC000		//   BG size 64 x 64 tiles
#define   LCD_COLOR16	0		//   BG 16 colors x 16 palettes
#define   LCD_COLOR256	0x0080		//   BG 256 colors x 1 palette
#define   LCD_BGCHAR(n)	(n << 2)	//   BG character base block #
#define   LCD_BGSCRN(n)	(n << 8)	//   BG screen base block #
#define LCD_BG0_HOFFSET	(IOBASE + 16)	// LCD BG0 Horizontal offset
#define LCD_BG0_VOFFSET	(IOBASE + 18)	// LCD BG0 Vertical offset
#define LCD_BG1_HOFFSET	(IOBASE + 20)	// LCD BG1 Horizontal offset
#define LCD_BG1_VOFFSET	(IOBASE + 22)	// LCD BG1 Vertical offset
#define LCD_BG2_HOFFSET	(IOBASE + 24)	// LCD BG2 Horizontal offset
#define LCD_BG2_VOFFSET	(IOBASE + 26)	// LCD BG2 Vertical offset
#define LCD_BG3_HOFFSET	(IOBASE + 28)	// LCD BG3 Horizontal offset
#define LCD_BG3_VOFFSET	(IOBASE + 30)	// LCD BG3 Vertical offset

typedef struct {
  hword* map;	// Map data pointer
  hword* tile;	// Tile data pointer
  int x, y;	// Position
 } screen;

#define KEY_STATUS	(IOBASE + 304)	// Key status
#define   KEY_L		0x200		//   L
#define   KEY_R		0x100		//   R
#define   KEY_DOWN	0x080		//   Down
#define   KEY_UP	0x040		//   Up
#define   KEY_LEFT	0x020		//   Left
#define   KEY_RIGHT	0x010		//   Right
#define   KEY_START	0x008		//   Start
#define   KEY_SELECT	0x004		//   Select
#define   KEY_B		0x002		//   B
#define   KEY_A		0x001		//   A
#define   KEY_ALL	0x3FF		// All key bits

#endif
