//
// k e y s c a n . c
//
// Key scan program
//
// May 7, 2003 by Wataru Nishida (http://www.skyfree.org)
//

#include "gba.h"
#include "oranges.pal"
#include "oranges.til"
#include "8x8.til"

hword* bcolor = (hword*) BG_PALETTE;

// Screen information

screen scrn[ 3 ] = {
  { (hword*) VRAM_SCRN(21), (hword*) VRAM_CHAR(3), 0, 0 },	// Page 0
  { (hword*) VRAM_SCRN(22), (hword*) VRAM_CHAR(3), 0, 0 },	// Page 1
  { (hword*) VRAM_SCRN(23), (hword*) VRAM_CHAR(0), 0, 0 },	// Page 2
 };

int page = 1;		// Page number
int hits = 0;		// Total hit counts

void init_screen(void) {
  int i, j, row, col, bit, val;

  // Initialize palettes

  for (i = 0; i < LCD_PAL_COLORS; i++)
    bcolor[ i ] = bg_palette[ i ];

  // Setup tiles for ASCII codes

  for (i = 0; i < 256; i++) {
    for (row = 0; row < 8; row++) {
      for (col = 7; col >= 0; col--) {
        bit = (char8x8[ i ][ row ] & (1 << col)) ? 1 : 0;
        if (col % 2)
          val = bit;
        else
          scrn[ 0 ].tile[(i * LCD_TILE_HWORD) + (row * 4) + ((7-col) / 2)] = \
          val + (bit << 8);
       }
     }
   }

  // Setup tiles for background

  for (i = 0; i < (BG_WIDTH * BG_HEIGHT + 1); i++) {
    for (j = 0; j < LCD_TILE_HWORD; j++) {
      scrn[ 2 ].tile[ (i * LCD_TILE_HWORD) + j ] = bg_tiles[ i ][ j ];
     }
   }

  // Arrange the background

  for (i = 0; i < BG_HEIGHT; i++)
    for (j = 0; j < BG_WIDTH; j++)
      scrn[ 2 ].map[ i * LCD_VWIDTH + j ] = (i * BG_WIDTH) + j;

  // Start LCD display

  register(LCD_BG0)  = LCD_SIZE3232  | LCD_COLOR256 |
                       LCD_BGCHAR(3) | LCD_BGSCRN(21);
  register(LCD_BG1)  = LCD_SIZE3232  | LCD_COLOR256 |
                       LCD_BGCHAR(3) | LCD_BGSCRN(22);
  register(LCD_BG2)  = LCD_SIZE3232  | LCD_COLOR256 |
                       LCD_BGCHAR(0) | LCD_BGSCRN(23);
  register(LCD_CTRL) = LCD_BG0EN | LCD_BG1EN | LCD_BG2EN | LCD_MODE0;
 }

void selectpage(int num) {
  if (num < 0)
    page = 0;
  else if (num >= 4)
    page = 3;
  else
    page = num;
 }

void locate(int xx, int yy) {
  if (xx < 0)
    scrn[ page ].x = 0;
  else if (xx >= LCD_RWIDTH)
    scrn[ page ].x = LCD_RWIDTH - 1;
  else
    scrn[ page ].x = xx;

  if (yy < 0)
    scrn[ page ].y = 0;
  else if (yy >= LCD_RHEIGHT)
    scrn[ page ].y = LCD_RHEIGHT - 1;
  else
    scrn[ page ].y = yy;
 }

void scrollup(void) {
  int i, j;

  for (i = 1; i < LCD_RHEIGHT; i++) {
    for (j = 0; j < LCD_RWIDTH; j++)
      scrn[ page ].map[ (i - 1) * LCD_VWIDTH + j ] = \
      scrn[ page ].map[ i * LCD_VWIDTH + j ];
   }
  for (j = 0; j < LCD_RWIDTH; j++)
    scrn[ page ].map[ 19 * LCD_VWIDTH + j ] = 0x20;
 }

void prints(const char* str) {
  const char* ptr = str;

  while (*ptr) {
    if (*ptr == '\n') {
      scrn[ page ].x = 0;
      if (scrn[ page ].y < (LCD_RHEIGHT - 1))
        scrn[ page ].y++;
      else
        scrollup();
      ptr++;
      continue;
     }
    scrn[ page ].map[ scrn[ page ].y * LCD_VHEIGHT + scrn[ page ].x ] = *ptr++;
    if (scrn[ page ].x < (LCD_RWIDTH - 1))
      scrn[ page ].x++;
    else {
      scrn[ page ].x = 0;
      if (scrn[ page ].y < (LCD_RHEIGHT - 1))
        scrn[ page ].y++;
      else
        scrollup();
     }
   }
 }

void print_hex8(int val) {
  char buf[ 3 ];

  buf[ 0 ] = "0123456789ABCDEF"[ (val & 0xF0) >> 4 ];
  buf[ 1 ] = "0123456789ABCDEF"[ val & 0x0F ];
  buf[ 2 ] = 0;
  prints(buf);
 }

void print_hex16(int val) {
  print_hex8((val & 0xFF00) >> 8);
  print_hex8(val & 0x00FF);
 }

int main() {
  int key, prev;

  // Initialize text screen

  init_screen();

  // Loop forever

  prints("Hit any key, please.\n");
  while (1) {
    key = register(KEY_STATUS);
    if ((key & KEY_ALL) == KEY_ALL) {
      prev = -1;
      continue;
     }
    else if (key == prev)
      continue;
    prev = key;
    key = key ^ KEY_ALL;
    prints("\nKey pressed: ");
    print_hex16(key);
    hits++;

    selectpage(0);
    locate(20, 1);
    prints("Hit = ");
    print_hex16(hits);
    selectpage(1);
   };
 }
