//
// m a k e p a l . c
//
// Prepare a palette table from index color table
// Intended for Adobe Color Table file (.act)
//
// May 6, 2003 by Wataru Nishida (http://www.skyfree.org)
//

#include <stdio.h>	// fprintf()
#include <stdlib.h>	// exit()
#include <unistd.h>	// open()
#include <fcntl.h>	// O_RDONLY
#include "gba.h"

#define DIM_NAME	"bg_palette"

int fd;		// Index file descriptor

void usage(void) {
  fprintf(stderr, "Usage: makepal index_table_file\n");
 }

void die(const char* msg) {
  fprintf(stderr, "%s\n", msg);
  exit(1);
 }

int readbyte(void) {
  unsigned char buf[ 3 ];
  int ret;

  ret = read(fd, buf, 1);
  if (ret != 1)
    die("Error: index file read failure!");
  return((int) buf[ 0 ]);
 }

int main (int argc, char** argv) {
  int i, r, g, b, gba;

  if (argc != 2) {
    usage();
    return 1;
   }

  fd = open(argv[ 1 ], O_RDONLY);
  if (fd < 0)
    die("Error: index file open failure!");

  printf("unsigned short int %s[] = {\n", DIM_NAME);
  for (i = 0; i < LCD_PAL_COLORS; i++) {
    if ((i % 8) == 0)
      printf("  ");
    r = readbyte() >> 3;
    g = readbyte() >> 3;
    b = readbyte() >> 3;
    gba = (b << 10) + (g << 5) + r;
    printf("0x%04X", gba);
    if (i == 255)
      printf("\n");
    else if ((i % 8) == 7)
      printf(",\n");
    else
      printf(", ");
   }
  close(fd);
  printf(" };\n");

  return 0;
 }
