//
// m a k e t i l e . c
//
// Synthesize tiles from index colored image file
//
// May 6, 2003 by Wataru Nishida (http://www.skyfree.org)
//

#include <stdio.h>	// fprintf(), malloc()
#include <stdlib.h>	// atoi(), exit()
#include <unistd.h>	// open()
#include <fcntl.h>	// O_RDONLY
#include "gba.h"

#define DIM_NAME	"bg_tiles"

void usage(void) {
  fprintf(stderr, "Usage: maketile image_file width height\n");
  fprintf(stderr, "         image_file -- index colored image file\n");
  fprintf(stderr, "         width  -- number of horizontal tiles\n");
  fprintf(stderr, "         height -- number of vertical tiles\n");
 }

void die(const char* msg) {
  fprintf(stderr, "%s\n", msg);
  exit(1);
 }

int main (int argc, char** argv) {
  int fd, width, height, ret, i, j, row, col, cnt = 0, idx;
  unsigned char* screen;

  if (argc != 4) {
    usage();
    return 1;
   }
  width  = atoi(argv[ 2 ]);
  height = atoi(argv[ 3 ]);

  fd = open(argv[ 1 ], O_RDONLY);
  if (fd < 0)
    die("Error: index file open failure!");

  screen = (unsigned char*) malloc(width * height * 64);
  if (screen == NULL)
    die("Error: Insufficient memory");
  ret = read(fd, screen, width * height * 64);
  if (ret != width * height * 64)
    die("Error: Image file read failure");

  printf("#define BG_WIDTH\t%d\n", width);
  printf("#define BG_HEIGHT\t%d\n", height);
  printf("#define BG_BLANK\t(BG_WIDTH * BG_HEIGHT)\n\n");
  printf("unsigned short int %s[][32] = {\n", DIM_NAME);
  for (i = 0; i < height; i++) {
    for (j = 0; j < width; j++) {
      printf("  { ");
      for (row = 0; row < 8; row++) {
        for (col = 0; col < 4; col++) {
          idx = (i * width * 64) + (j * 8) + \
                (row * width * 8) + (col * 2);
          printf("0x%04X", (screen[ idx + 1 ] << 8) + screen[ idx ]);
          if (! (col == 3 && row == 7))
            printf(", ");
         }
        if ((row % 2) == 1) {
          if (row == 7)
            printf(" }, // %d\n", cnt++);
          else
            printf("\n    ");
         }
       }
     }
   }
  printf("  { 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0," \
         "0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0 } // %d\n", cnt);
  close(fd);
  printf(" };\n");

  return 0;
 }
